@@ -247,13 +247,18 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
         def get_content(self) -> Optional[Union[bytes, bytearray, str]]:
 
             content = None
-            content_length = int(self.headers.get('content-length'))
+            content_length = self.headers.get('content-length')
+
             if content_length is None:
                 self.send_error_response(
                     HTTPStatus.LENGTH_REQUIRED,
                     "Missing Content-Length header")
+                return None
 
-            elif content_length > PATCH_SIZE_LIMIT:
+            else:
+                content_length = int(content_length)
+
+            if content_length > PATCH_SIZE_LIMIT:
                 self.send_error_response(
                     HTTPStatus.REQUEST_ENTITY_TOO_LARGE,
                     "Patch is too large")
@@ -315,6 +320,10 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
                 body = ''.join(resp.tbe.format())
 
                 if isinstance(resp.exception,
+                              (DE1DBNoMatchingRecord,)):
+                    http_status = HTTPStatus.NOT_FOUND
+
+                elif isinstance(resp.exception,
                               (DE1APIUnsupportedStateTransitionError,
                                DE1NotConnectedError,
                                DE1IsConnectedError,
@@ -457,10 +466,11 @@ def run_api_inbound(log_queue: multiprocessing.Queue,
             if resource is None:
                 return
 
-            if resource is not Resource.DE1_PROFILE:
+            if resource not in (Resource.DE1_PROFILE,
+                                Resource.DE1_PROFILE_ID):
                 self.send_error_response(
                     HTTPStatus.NOT_IMPLEMENTED,
-                    f"PUT not yet supported beyond {Resource.DE1_PROFILE}"
+                    f"PUT not yet supported for {resource}"
                 )
                 return
 
