@@ -19,6 +19,7 @@ from typing import Union, Dict, Coroutine, Optional, List, Callable
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.scanner import AdvertisementData
+from bleak.backends.service import BleakGATTServiceCollection
 
 from pyDE1.bleak_client_wrapper import BleakClientWrapped
 
@@ -39,7 +40,7 @@ from pyDE1.de1.c_api import \
 from pyDE1.de1.ble import CUUID
 from pyDE1.de1.notifications import NotificationState, MMR0x80Data
 from pyDE1.de1.profile import Profile, ProfileByFrames, \
-    DE1ProfileValidationError
+    DE1ProfileValidationError, SourceFormat
 
 from pyDE1.i_target_setter import I_TargetSetter
 from pyDE1.event_manager.events import ConnectivityState, ConnectivityChange
@@ -131,6 +132,9 @@ class DE1 (Singleton):
         # Internal flag
         self._recorder_active = False
 
+        self._reconnect_delay = 0
+        self._logging_reconnect = True
+
         # Used for volume estimation at this time
         asyncio.create_task(self._event_shot_sample.subscribe(
             self._create_self_callback_ssu()))
@@ -253,15 +257,19 @@ class DE1 (Singleton):
     async def _notify_ready(self):
         self._ready.set()
         await self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.READY))
+            self._connectivity_change(arrival_time=time.time(),
+                                      state=ConnectivityState.READY))
         logger.info("Ready")
 
     async def _notify_not_ready(self):
         self._ready.clear()
         await self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.NOT_READY))
+            self._connectivity_change(arrival_time=time.time(),
+                                      state=ConnectivityState.NOT_READY))
+
+    @property
+    def is_ready(self):
+        return self._ready.is_set()
 
     @classmethod
     def device_adv_is_recognized_by(cls, device: BLEDevice, adv: AdvertisementData):
@@ -292,6 +300,15 @@ class DE1 (Singleton):
                 self._name = address.name
             self._bleak_client = BleakClientWrapped(self._address_or_bledevice)
 
+    # Helper method to populate a ConnectivityChange
+
+    def _connectivity_change(self, arrival_time: float,
+                             state: ConnectivityState):
+        return ConnectivityChange(arrival_time=arrival_time,
+                                  state=state,
+                                  id=self.address,
+                                  name=self.name)
+
     #
     # Self-contained calls for API
     #
@@ -389,56 +406,69 @@ class DE1 (Singleton):
         logger.info(f"Connecting to DE1 at {self.address}")
 
         assert self._bleak_client is not None
-        self._bleak_client.set_disconnected_callback(
-            self._create_disconnect_callback()
-        )
 
-        await asyncio.gather(self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.CONNECTING)),
-            self._bleak_client.connect(timeout=timeout),
-        )
+        if not self.is_connected:
 
-        if self.is_connected:
-            if self.name is None:
-                self._name = self._bleak_client.name
-            logger.info(f"Connected to DE1 at {self.address}")
-            await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.CONNECTED))
-            # This can take some time, potentially delaying scale connection
-            # At least BlueZ doesn't like concurrent connection requests
-            asyncio.create_task(self.initialize_after_connection())
+            self._bleak_client.set_disconnected_callback(
+                self._create_disconnect_callback()
+            )
+            await asyncio.gather(self._event_connectivity.publish(
+                self._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.CONNECTING)),
+                self._bleak_client.connect(timeout=timeout),
+                return_exceptions=True
+            )
 
-        else:
-            logger.error(f"Connection failed to DE1 at {self.address}")
-            await self._notify_not_ready()
-            await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.DISCONNECTED))
+            if self.is_connected:
+                if self.name is None:
+                    self._name = self._bleak_client.name
+                logger.info(f"Connected to DE1 at {self.address}")
+                await self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.CONNECTED))
+                # This can take time, potentially delaying scale connection
+                # At least BlueZ doesn't like concurrent connection requests
+                asyncio.create_task(self.initialize_after_connection())
+
+            else:
+                logger.error(f"Connection failed to DE1 at {self.address}")
+                await self._notify_not_ready()
+                await self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTED))
 
     async def disconnect(self):
         logger.info(f"Disconnecting from DE1")
         if self._bleak_client is None:
             logger.info(f"Disconnecting from DE1, no client")
             return
-        await asyncio.gather(
-            self._notify_not_ready(),
-            self._bleak_client.disconnect(),
-            self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.DISCONNECTING))
-        )
+
         if self.is_connected:
-            logger.error(f"Disconnect failed from DE1 at {self.address}")
-            await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.CONNECTED))
-        else:
-            logger.info(f"DE1.disconnect(): Disconnected from DE1 at {self.address}")
-            await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.DISCONNECTED))
+            await asyncio.gather(
+                self._notify_not_ready(),
+                self._bleak_client.disconnect(),
+                self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTING)),
+                return_exceptions=True
+            )
+            if self.is_connected:
+                logger.error(
+                    f"Disconnect failed from DE1 at {self.address}")
+                await self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.CONNECTED))
+            else:
+                logger.info("DE1.disconnect(): Disconnected from DE1 at "
+                            f"{self.address}")
+                await self._event_connectivity.publish(
+                    self._connectivity_change(arrival_time=time.time(),
+                                              state=ConnectivityState.DISCONNECTED))
 
     # TODO: Decide how to handle  self._disconnected_callback
     #   disconnected_callback (callable): Callback that will be scheduled in the
@@ -464,14 +494,45 @@ class DE1 (Singleton):
             asyncio.ensure_future(asyncio.gather(
                 self._notify_not_ready(),
                 de1._event_connectivity.publish(
-                    ConnectivityChange(arrival_time=time.time(),
-                                       state=ConnectivityState.DISCONNECTED)),
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTED)),
+                return_exceptions=True
             ))
             # TODO: if not shutdown underway:
             de1.prepare_for_connection(wipe_address=client.willful_disconnect)
+            if not client.willful_disconnect:
+                # await self._bleak_client.disconnect()
+                asyncio.get_event_loop().create_task(self._reconnect())
 
         return disconnect_callback
 
+    async def _reconnect(self):
+        """
+        Will try immediately, then 1, 2, 3, ..., 10, 10, ... seconds later
+        """
+        # Workaround for https://github.com/hbldh/bleak/issues/376
+        self._bleak_client.services = BleakGATTServiceCollection()
+        class_name = type(self).__name__
+        if self._logging_reconnect:
+            logger.info(
+                f"Will try reconnecting to {class_name} at {self.address} "
+                f"after waiting {self._reconnect_delay} seconds.")
+        await asyncio.sleep(self._reconnect_delay)
+
+        await self.connect()
+        if self.is_connected:
+            self._reconnect_delay = 0
+            self._logging_reconnect = True
+        else:
+            if self._reconnect_delay <= 10:
+                self._reconnect_delay = self._reconnect_delay +1
+            if self._reconnect_delay == 10:
+                logger.info("Suppressing further reconnect messages. "
+                            "Will keep trying at 10-second intervals.")
+                self._logging_reconnect = False
+            asyncio.get_event_loop().create_task(
+                self._reconnect(), name='ReconnectDE1')
 
     @property
     def is_connected(self):
@@ -576,9 +637,10 @@ class DE1 (Singleton):
                 wait_for = None
 
         elif cuuid.can_read and cuuid not in (
-                CUUID.WriteToMMR,   # Decode not implemented
-                CUUID.HeaderWrite,  # Decode not implemented
-                CUUID.FrameWrite):  # Decode not implemented
+                CUUID.RequestedState,   # Comes back in StateInfo
+                CUUID.WriteToMMR,       # Decode not implemented
+                CUUID.HeaderWrite,      # Decode not implemented
+                CUUID.FrameWrite):      # Decode not implemented
             wait_for = self.read_cuuid(cuuid)
 
         else:
@@ -1208,9 +1270,9 @@ class DE1 (Singleton):
     async def connectivity_setter(self, value):
         assert isinstance(value, ConnectivityEnum), \
             f"mode of {value} not a ConnectivityEnum "
-        if value is ConnectivityEnum.CONNECTED and not self.is_connected:
+        if value is ConnectivityEnum.CONNECTED:
             await self.connect()
-        elif value is ConnectivityEnum.NOT_CONNECTED and self.is_connected:
+        elif value is ConnectivityEnum.NOT_CONNECTED:
             await self.disconnect()
         else:
             raise DE1APIValueError(
@@ -1251,7 +1313,7 @@ class DE1 (Singleton):
             await self.read_one_mmr0x80_and_wait(MMR0x80LowAddr.GHC_INFO)
 
         cs = self.current_state
-        if cs is API_MachineStates.NoRequest:
+        if cs == API_MachineStates.NoRequest:
             logger.warning(f"Refreshing current state as is NoRequest")
             await self.read_cuuid(CUUID.StateInfo)
             cs = self.current_state
@@ -1260,17 +1322,17 @@ class DE1 (Singleton):
                      f"while in {API_MachineStates(cs).name}")
 
         if mode is DE1ModeEnum.SLEEP:
-            if cs is API_MachineStates.Idle:
+            logger.debug(f"current state: {cs}, {type(cs)}")
+            if cs == API_MachineStates.Idle:
                 logger.debug("API triggered sleep()")
                 await self.sleep()
-            if self.current_state in (API_MachineStates.Sleep,
-                                      API_MachineStates.GoingToSleep):
+            elif self.current_state in (API_MachineStates.Sleep,
+                                        API_MachineStates.GoingToSleep):
                 pass
             else:
                 raise DE1APIUnsupportedStateTransitionError(mode, cs, css)
 
         elif mode is DE1ModeEnum.WAKE:
-            logger.debug(f"In WAKE, cs: {API_MachineStates(cs).name}")
             if cs in (API_MachineStates.Sleep,
                       API_MachineStates.GoingToSleep):
                 logger.debug("API triggered idle()")
@@ -1300,7 +1362,7 @@ class DE1 (Singleton):
 
         elif mode is DE1ModeEnum.SKIP_TO_NEXT:
 
-            if cs is API_MachineStates.Espresso \
+            if cs == API_MachineStates.Espresso \
                 and self.current_substate in (API_Substates.PreInfuse,
                                               API_Substates.Pour):
                 logger.debug("API triggered skip_to_next()")
@@ -1319,7 +1381,7 @@ class DE1 (Singleton):
                     f"DE1 does not permit {mode} unless GHC is not installed."
                 )
 
-            if cs is not API_MachineStates.Idle:
+            if cs != API_MachineStates.Idle:
                 raise DE1APIUnsupportedStateTransitionError(mode, cs, css)
 
             if mode is DE1ModeEnum.ESPRESSO:
@@ -1341,3 +1403,30 @@ class DE1 (Singleton):
         else:
             raise DE1APIUnsupportedStateTransitionError(mode, cs, css)
 
+    @property
+    def profile_id(self):
+        if self._latest_profile is not None:
+            return self._latest_profile.id
+        else:
+            return None
+
+    async def set_profile_by_id(self, pid: str):
+
+        async with aiosqlite.connect('/var/lib/pyDE1/pyDE1.sqlite3') as db:
+            cur: aiosqlite.Cursor = await db.execute(
+                'SELECT source, source_format FROM profile '
+                'WHERE id == :id', (pid,)
+            )
+            row = await cur.fetchone()
+
+        if row is None:
+            raise DE1DBNoMatchingRecord(
+                f"No profile record for {pid}")
+        (source, source_format) = row
+        if source_format != SourceFormat.JSONv2.value:
+            raise DE1APITypeError(
+                f"Only JSONv2 profiles supported, not {source_format}"
+            )
+
+        await self.upload_json_v2_profile(source)
+
