@@ -6,6 +6,6 @@ GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
 
-from .scale import Scale, \
+from .scale import Scale, scale_factory, recognized_scale_prefixes, \
     ScaleError, ScaleNoAddressError, ScaleNotConnectedError
 from .atomax_skale_ii import AtomaxSkaleII
\ No newline at end of file
