@@ -15,8 +15,10 @@ from typing import Optional, Callable, Coroutine, Union
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.scanner import AdvertisementData
+from bleak.backends.service import BleakGATTServiceCollection
 
 from pyDE1.bleak_client_wrapper import BleakClientWrapped
+from pyDE1.de1.c_api import API_MachineStates
 from pyDE1.scanner import BleakScannerWrapped
 from pyDE1.exceptions import DE1APIValueError, DE1NoAddressError
 from pyDE1.dispatcher.resource import ConnectivityEnum
@@ -26,6 +28,8 @@ from pyDE1.scale.events import ScaleWeightUpdate, ScaleTareSeen
 from pyDE1.scanner import _registered_ble_prefixes, DiscoveredDevices
 from pyDE1.config.bluetooth import CONNECT_TIMEOUT
 
+from pyDE1.de1 import DE1
+
 logger = logging.getLogger('Scale')
 
 # TODO: Can/should this be related to the class?
@@ -84,10 +88,14 @@ class Scale:
 
         self._ready = asyncio.Event()
 
+        self._reconnect_delay = 0
+        self._logging_reconnect = True
+
         asyncio.get_event_loop().create_task(
             self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.NOT_READY)))
+                self._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.NOT_READY)))
 
         self._estimated_period = self._nominal_period
         self._last_weight_update_received = 0
@@ -157,39 +165,44 @@ class Scale:
         logger.info(f"Connecting to {class_name} at {self.address}")
 
         assert self._bleak_client is not None
-        self._bleak_client.set_disconnected_callback(
-            self._create_disconnect_callback()
-        )
 
-        await asyncio.gather(self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.CONNECTING)),
-            self._bleak_client.connect(timeout=timeout),
-        )
+        if not self.is_connected:
 
-        if self.is_connected:
-            self._address_or_bledevice = self._bleak_client.address
-            if self.name is None:
-                self._name = self._bleak_client.name
-            logger.info(f"Connected to {class_name} at {self.address}")
-            await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.CONNECTED))
-            # This can take some time, potentially delaying DE1 connection
-            # At least BlueZ doesn't like concurrent connection requests
-            asyncio.create_task(self.standard_initialization())
+            self._bleak_client.set_disconnected_callback(
+                self._create_disconnect_callback()
+            )
 
-            # TODO: Does the ScaleProcessor get properly reset?
+            await asyncio.gather(self._event_connectivity.publish(
+                self._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.CONNECTING)),
+                self._bleak_client.connect(timeout=timeout),
+                return_exceptions=True
+            )
 
-        else:
-            logger.error(
-                f"Connection failed to {class_name} at {self.address}")
-            await asyncio.gather(self._notify_not_ready,
-                                 self._event_connectivity.publish(
-                                     ConnectivityChange(
-                                         arrival_time=time.time(),
-                                         state=ConnectivityState.DISCONNECTED))
-                                 )
+            if self.is_connected:
+                self._address_or_bledevice = self._bleak_client.address
+                if self.name is None:
+                    self._name = self._bleak_client.name
+                logger.info(f"Connected to {class_name} at {self.address}")
+                await self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.CONNECTED))
+                # This can take some time, potentially delaying DE1 connection
+                # At least BlueZ doesn't like concurrent connection requests
+                asyncio.create_task(self.standard_initialization())
+
+                # TODO: Does the ScaleProcessor get properly reset?
+
+            else:
+                logger.error(
+                    f"Connection failed to {class_name} at {self.address}")
+                await self._notify_not_ready()
+                await self._event_connectivity.publish(
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTED))
 
     async def standard_initialization(self, hold_notification=False):
         """
@@ -206,15 +219,30 @@ class Scale:
     async def _notify_ready(self):
         self._ready.set()
         await self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.READY))
+            self._connectivity_change(
+                arrival_time=time.time(),
+                state=ConnectivityState.READY))
         logger.info("Ready")
 
     async def _notify_not_ready(self):
         self._ready.clear()
         await self._event_connectivity.publish(
-            ConnectivityChange(arrival_time=time.time(),
-                               state=ConnectivityState.NOT_READY))
+            self._connectivity_change(
+                arrival_time=time.time(),
+                state=ConnectivityState.NOT_READY))
+
+    @property
+    def is_ready(self):
+        return self._ready.is_set()
+
+    # Helper method to populate a ConnectivityChange
+
+    def _connectivity_change(self, arrival_time: float,
+                             state: ConnectivityState):
+        return ConnectivityChange(arrival_time=arrival_time,
+                                  state=state,
+                                  id=self.address,
+                                  name=self.name)
 
     async def disconnect(self):
         class_name = type(self).__name__
@@ -222,29 +250,33 @@ class Scale:
         if self._bleak_client is None:
             logger.info(f"Disconnecting from {class_name}, no client")
             return
+
         if self.is_connected:
-            logger.debug(f"await gather disconnect, not ready, disconnecting")
-            gr = await asyncio.gather(
+            await asyncio.gather(
                 self._bleak_client.disconnect(),
                 self._notify_not_ready(),
                 self._event_connectivity.publish(
-                    ConnectivityChange(arrival_time=time.time(),
-                                       state=ConnectivityState.DISCONNECTING))
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTING)),
+                return_exceptions=True
             )
 
         if self.is_connected:
             logger.error(
                 f"Disconnect failed from {class_name} at {self.address}")
             await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.CONNECTED))
+                self._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.CONNECTED))
         else:
             logger.info(
                 f"Scale.disconnect(): Disconnected from {class_name} "
                 f"at {self.address}")
             await self._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.DISCONNECTED))
+                self._connectivity_change(
+                    arrival_time=time.time(),
+                    state=ConnectivityState.DISCONNECTED))
 
     # TODO: Decide how to handle  self._disconnected_callback
 
@@ -427,12 +459,52 @@ class Scale:
             asyncio.ensure_future(asyncio.gather(
                 self._notify_not_ready(),
                 scale._event_connectivity.publish(
-                ConnectivityChange(arrival_time=time.time(),
-                                   state=ConnectivityState.DISCONNECTED)))
+                    self._connectivity_change(
+                        arrival_time=time.time(),
+                        state=ConnectivityState.DISCONNECTED)),
+                return_exceptions=True)
             )
+            # TODO: Don't try to reconnect on shutdown
+            if not client.willful_disconnect:
+                asyncio.get_event_loop().create_task(self._reconnect())
 
         return disconnect_callback
 
+    def _reset_reconnect(self):
+        self._reconnect_delay = 0
+        self._logging_reconnect = True
+
+    async def _reconnect(self):
+        """
+        Will try immediately, then 1, 2, 3, ..., 10, 10, ... seconds later
+        Each try includes default scan time (10 sec), then a delay
+        TODO: Is there a better pattern?
+        """
+        if DE1().current_state == API_MachineStates.Sleep:
+            logger.info("DE1 is sleeping, not retrying to connect")
+            return
+        # Workaround for https://github.com/hbldh/bleak/issues/376
+        self._bleak_client.services = BleakGATTServiceCollection()
+        class_name = type(self).__name__
+        if self._logging_reconnect:
+            logger.info(
+                f"Will try reconnecting to {class_name} at {self.address} "
+                f"after waiting {self._reconnect_delay} seconds.")
+        await asyncio.sleep(self._reconnect_delay)
+
+        await self.connect()
+        if self.is_connected:
+            self._reset_reconnect()
+        else:
+            if self._reconnect_delay <= 10:
+                self._reconnect_delay = self._reconnect_delay +1
+            if self._reconnect_delay == 10:
+                logger.info("Suppressing further reconnect messages. "
+                            "Will keep trying at 10-second intervals.")
+                self._logging_reconnect = False
+            asyncio.get_event_loop().create_task(
+                self._reconnect(), name='ReconnectScale')
+
     @property
     def nominal_period(self):
         return self._nominal_period
@@ -505,18 +577,6 @@ class Scale:
                 retval = ConnectivityEnum.CONNECTED
         return retval
 
-    async def connectivity_setter(self, value):
-        assert isinstance(value, ConnectivityEnum), \
-            f"mode of {value} not a ConnectivityEnum"
-        if value is ConnectivityEnum.CONNECTED and not self.is_connected:
-            await self.connect()
-        elif value is ConnectivityEnum.NOT_CONNECTED and self.is_connected:
-            await self.disconnect()
-        else:
-            raise DE1APIValueError(
-                "Only CONNECTED and NOT_CONNECTED can be set, "
-                f"not {value}")
-
     @staticmethod
     def register_constructor(constructor: Callable, prefix: str):
         _prefix_to_constructor[prefix] = constructor
